@@ -43,11 +43,9 @@ const GtkClutter = imports.gi.GtkClutter;
 
 const _PDF_LOADER_TIMEOUT = 400;
 
-function ViewEmbed() {
-    this._init();
-}
+const ViewEmbed = new Lang.Class({
+    Name: 'ViewEmbed',
 
-ViewEmbed.prototype  = {
     _init: function() {
         this._adjustmentValueId = 0;
         this._adjustmentChangedId = 0;
@@ -68,7 +66,7 @@ ViewEmbed.prototype  = {
             Clutter.BinAlignment.FILL, Clutter.BinAlignment.FILL);
 
         // pack the toolbar
-        this._toolbar = new MainToolbar.MainToolbar();
+        this._toolbar = new MainToolbar.OverviewToolbar();
         this._contentsActor.add_actor(this._toolbar.actor);
         this._contentsLayout.set_fill(this._toolbar.actor, true, false);
 
@@ -103,7 +101,9 @@ ViewEmbed.prototype  = {
         // create the dropdown for the search bar, it's hidden by default
         this._dropdownBox = new Searchbar.Dropdown();
         this._overlayLayout.add(this._dropdownBox.actor,
-            Clutter.BinAlignment.CENTER, Clutter.BinAlignment.START);
+            Clutter.BinAlignment.CENTER, Clutter.BinAlignment.FIXED);
+        this._dropdownBox.actor.add_constraint(new Clutter.BindConstraint({ source: this._toolbar.toolbarActor,
+                                                                            coordinate: Clutter.BindCoordinate.Y }));
 
         // create the OSD toolbar for selected items, it's hidden by default
         this._selectionToolbar = new Selections.SelectionToolbar();
@@ -379,4 +379,4 @@ ViewEmbed.prototype  = {
         let errorBox = new ErrorBox.ErrorBox(message, exception.message);
         this._scrolledWinPreview.add_with_viewport(errorBox.widget);
     }
-};
+});
