@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Red Hat, Inc.
+ * Copyright (c) 2011, 2012 Red Hat, Inc.
  *
  * Gnome Documents is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by the
@@ -62,8 +62,14 @@ GdkPixbuf *gd_embed_image_in_frame (GdkPixbuf *source_image,
                                     GtkBorder *slice_width,
                                     GtkBorder *border_width);
 
+const char *gd_filename_get_extension_offset (const char *filename);
+
 char *gd_filename_strip_extension (const char * filename_with_extension);
 
+const char *gd_filename_to_mime_type (const gchar *filename_with_extension);
+
+const char *gd_filename_to_rdf_type (const gchar *filename_with_extension);
+
 gboolean gd_time_val_from_iso8601 (const gchar *string,
                                    GTimeVal *timeval);
 gchar *gd_iso8601_from_timestamp (gint64 timestamp);
