@@ -31,7 +31,6 @@ const Mainloop = imports.mainloop;
 const Config = imports.config;
 const Embed = imports.embed;
 const Global = imports.global;
-const Searchbar = imports.searchbar;
 const Selections = imports.selections;
 const Utils = imports.utils;
 const WindowMode = imports.windowMode;
@@ -40,11 +39,9 @@ const _ = imports.gettext.gettext;
 
 const _CONFIGURE_ID_TIMEOUT = 100; // msecs
 
-function MainWindow(app) {
-    this._init(app);
-}
+const MainWindow = new Lang.Class({
+    Name: 'MainWindow',
 
-MainWindow.prototype = {
     _init: function(app) {
         this._configureId = 0;
 
@@ -91,27 +88,12 @@ MainWindow.prototype = {
 
         Global.modeController.connect('fullscreen-changed',
                                       Lang.bind(this, this._onFullscreenChanged));
-        Global.modeController.connect('window-mode-changed',
-                                      Lang.bind(this, this._onWindowModeChanged));
-
-        // the base layout is a vertical ClutterBox
-        this._clutterBoxLayout = new Clutter.BoxLayout({ vertical: true });
-        this._clutterBox = new Clutter.Box({ layout_manager: this._clutterBoxLayout });
-        this._clutterBox.add_constraint(
-            new Clutter.BindConstraint({ coordinate: Clutter.BindCoordinate.SIZE,
-                                         source: stage }));
-
-        stage.add_actor(this._clutterBox);
-
-        // first child: searchbar filling the X axis
-        this._searchbar = new Searchbar.Searchbar();
-        this._clutterBox.add_actor(this._searchbar.actor);
-        this._clutterBoxLayout.set_fill(this._searchbar.actor, true, false);
 
         this._embed = new Embed.ViewEmbed();
-        this._clutterBox.add_actor(this._embed.actor);
-        this._clutterBoxLayout.set_expand(this._embed.actor, true);
-        this._clutterBoxLayout.set_fill(this._embed.actor, true, true);
+        this._embed.actor.add_constraint(
+            new Clutter.BindConstraint({ coordinate: Clutter.BindCoordinate.SIZE,
+                                         source: stage }));
+        stage.add_actor(this._embed.actor);
     },
 
     _saveWindowGeometry: function() {
@@ -158,11 +140,6 @@ MainWindow.prototype = {
         Global.settings.set_boolean('window-maximized', maximized);
     },
 
-    _onWindowModeChanged: function() {
-        if (Global.modeController.getWindowMode() == WindowMode.WindowMode.PREVIEW)
-            this._searchbar.hide();
-    },
-
     _onFullscreenChanged: function(controller, fullscreen) {
         if (fullscreen)
             this.window.fullscreen();
@@ -190,13 +167,19 @@ MainWindow.prototype = {
         let keyval = event.get_keyval()[1];
         let state = event.get_state()[1];
         let fullscreen = Global.modeController.getFullscreen();
+        let direction = this.window.get_direction();
 
-        if (keyval == Gdk.KEY_f) {
+        if ((keyval == Gdk.KEY_f || keyval == Gdk.KEY_F11) &&
+            ((state & Gdk.ModifierType.CONTROL_MASK) == 0)) {
             Global.modeController.toggleFullscreen();
             return true;
         }
 
-        if (keyval == Gdk.KEY_Escape ||
+        if ((fullscreen && keyval == Gdk.KEY_Escape) ||
+            ((state & Gdk.ModifierType.MOD1_MASK) != 0 &&
+             (direction == Gtk.TextDirection.LTR && keyval == Gdk.KEY_Left) ||
+             (direction == Gtk.TextDirection.RTL && keyval == Gdk.KEY_Right)) ||
+            keyval == Gdk.KEY_BackSpace ||
             keyval == Gdk.KEY_Back) {
             Global.modeController.setWindowMode(WindowMode.WindowMode.OVERVIEW);
             return true;
@@ -209,11 +192,11 @@ MainWindow.prototype = {
         let keyval = event.get_keyval()[1];
 
         if (Utils.isSearchEvent(event)) {
-            this._searchbar.toggle();
+            this._embed._toolbar.searchbar.toggle();
             return true;
         }
 
-        if (this._searchbar.deliverEvent(event))
+        if (this._embed._toolbar.searchbar.deliverEvent(event))
             return true;
 
         if (Global.selectionController.getSelectionMode() &&
@@ -263,4 +246,4 @@ MainWindow.prototype = {
             aboutDialog.destroy();
         });
     }
-};
+});
