@@ -27,11 +27,9 @@ const Pango = imports.gi.Pango;
 const Lang = imports.lang;
 const Signals = imports.signals;
 
-function BaseManager(title) {
-    this._init(title);
-};
+const BaseManager = new Lang.Class({
+    Name: 'BaseManager',
 
-BaseManager.prototype = {
     _init: function(title) {
         this._items = {};
         this._activeItem = null;
@@ -164,7 +162,7 @@ BaseManager.prototype = {
 
         // TODO: merge existing item properties with new values
     }
-};
+});
 Signals.addSignalMethods(BaseManager.prototype);
 
 // GTK+ implementations
@@ -175,11 +173,9 @@ const BaseModelColumns = {
     HEADING_TEXT: 2
 };
 
-function BaseModel(manager) {
-    this._init(manager);
-}
+const BaseModel = new Lang.Class({
+    Name: 'BaseModel',
 
-BaseModel.prototype = {
     _init: function(manager) {
         this.model = Gd.create_item_store();
         this._manager = manager;
@@ -209,13 +205,11 @@ BaseModel.prototype = {
                               item.id, item.name, '');
         }
     }
-};
+});
 
-function BaseView(manager) {
-    this._init(manager);
-}
+const BaseView = new Lang.Class({
+    Name: 'BaseView',
 
-BaseView.prototype = {
     _init: function(manager) {
         this._model = new BaseModel(manager);
         this._manager = manager;
@@ -286,5 +280,5 @@ BaseView.prototype = {
         if (additionalFunc)
             additionalFunc(col, cell, model, iter);
     }
-};
+});
 Signals.addSignalMethods(BaseView.prototype);
