@@ -24,6 +24,7 @@ const Global = imports.global;
 const Gd = imports.gi.Gd;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const Lang = imports.lang;
 
 const QueryColumns = {
     URN: 0,
@@ -47,22 +48,18 @@ const QueryFlags = {
 
 const LOCAL_COLLECTIONS_IDENTIFIER = 'gd:collection:local:';
 
-function Query(sparql) {
-    this._init(sparql);
-}
+const Query = new Lang.Class({
+    Name: 'Query',
 
-Query.prototype = {
     _init: function(sparql) {
         this.sparql = sparql;
         this.activeSource = Global.sourceManager.getActiveItem();
     }
-};
+});
 
-function QueryBuilder() {
-    this._init();
-}
+const QueryBuilder = new Lang.Class({
+    Name: 'QueryBuilder',
 
-QueryBuilder.prototype = {
     _init: function() {
     },
 
@@ -262,4 +259,4 @@ QueryBuilder.prototype = {
 
         return new Query(sparql);
     }
-};
+});
