@@ -42,11 +42,9 @@ const Signals = imports.signals;
 const _COLLECTION_PLACEHOLDER_ID = 'collection-placeholder';
 
 // fetch all the collections a given item is part of
-function FetchCollectionsJob(urn) {
-    this._init(urn);
-}
+const FetchCollectionsJob = new Lang.Class({
+    Name: 'FetchCollectionsJob',
 
-FetchCollectionsJob.prototype = {
     _init: function(urn) {
         this._urn = urn;
         this._collections = [];
@@ -96,7 +94,7 @@ FetchCollectionsJob.prototype = {
         if (this._callback)
             this._callback(this._collections);
     }
-};
+});
 
 // fetch the state of every collection applicable to the selected items
 const OrganizeCollectionState = {
@@ -106,11 +104,9 @@ const OrganizeCollectionState = {
     HIDDEN: 1 << 2
 };
 
-function FetchCollectionStateForSelectionJob() {
-    this._init();
-}
+const FetchCollectionStateForSelectionJob = new Lang.Class({
+    Name: 'FetchCollectionStateForSelectionJob',
 
-FetchCollectionStateForSelectionJob.prototype = {
     _init: function() {
         this._collectionsForItems = {};
         this._runningJobs = 0;
@@ -193,14 +189,12 @@ FetchCollectionStateForSelectionJob.prototype = {
         if (this._callback)
             this._callback(collectionState);
     }
-};
+});
 
 // updates the mtime for the given resource to the current system time
-function UpdateMtimeJob(urn) {
-    this._init(urn);
-}
+const UpdateMtimeJob = new Lang.Class({
+    Name: 'UpdateMtimeJob',
 
-UpdateMtimeJob.prototype = {
     _init: function(urn) {
         this._urn = urn;
     },
@@ -221,14 +215,12 @@ UpdateMtimeJob.prototype = {
                     this._callback();
             }));
     }
-};
+});
 
 // adds or removes the selected items to the given collection
-function SetCollectionForSelectionJob(collectionUrn, setting) {
-    this._init(collectionUrn, setting);
-}
+const SetCollectionForSelectionJob = new Lang.Class({
+    Name: 'SetCollectionForSelectionJob',
 
-SetCollectionForSelectionJob.prototype = {
     _init: function(collectionUrn, setting) {
         this._collectionUrn = collectionUrn;
         this._setting = setting;
@@ -275,14 +267,12 @@ SetCollectionForSelectionJob.prototype = {
                 }));
         }
     }
-};
+});
 
 // creates an (empty) collection with the given name
-function CreateCollectionJob(name) {
-    this._init(name);
-}
+const CreateCollectionJob = new Lang.Class({
+    Name: 'CreateCollectionJob',
 
-CreateCollectionJob.prototype = {
     _init: function(name) {
         this._name = name;
         this._createdUrn = null;
@@ -315,7 +305,7 @@ CreateCollectionJob.prototype = {
                     this._callback(this._createdUrn);
             }));
     }
-};
+});
 
 const OrganizeModelColumns = {
     ID: 0,
@@ -323,11 +313,9 @@ const OrganizeModelColumns = {
     STATE: 2
 };
 
-function OrganizeCollectionModel() {
-    this._init();
-}
+const OrganizeCollectionModel = new Lang.Class({
+    Name: 'OrganizeCollectionModel',
 
-OrganizeCollectionModel.prototype = {
     _init: function() {
         this.model = Gd.create_organize_store();
         this._placeholderRef = null;
@@ -431,11 +419,15 @@ OrganizeCollectionModel.prototype = {
         }
     },
 
-    forgetPlaceholder: function() {
-        let path = this._placeholderRef.get_path();
-        this._placeholderRef = null;
+    getPlaceholder: function(forget) {
+        let ret = null;
 
-        return path;
+        if (this._placeholderRef)
+            ret = this._placeholderRef.get_path();
+        if (forget)
+            this._placeholderRef = null;
+
+        return ret;
     },
 
     destroy: function() {
@@ -449,14 +441,14 @@ OrganizeCollectionModel.prototype = {
             this._collRemovedId = 0;
         }
     }
-};
+});
 
-function OrganizeCollectionView() {
-    this._init();
-}
+const OrganizeCollectionView = new Lang.Class({
+    Name: 'OrganizeCollectionView',
 
-OrganizeCollectionView.prototype = {
     _init: function() {
+        this._choiceConfirmed = false;
+
         this._model = new OrganizeCollectionModel();
         this.widget = new Gtk.TreeView({ headers_visible: false,
                                          vexpand: true,
@@ -516,7 +508,7 @@ OrganizeCollectionView.prototype = {
             return;
         }
 
-        let path = this._model.forgetPlaceholder();
+        let path = this._model.getPlaceholder(true);
         if (!path)
             return;
 
@@ -527,7 +519,7 @@ OrganizeCollectionView.prototype = {
         job.run(null);
     },
 
-    _onTextEdited: function(cell, pathStr, newText) {
+    _onTextEditedReal: function(cell, path, newText) {
         cell.editable = false;
 
         if (!newText || newText == '') {
@@ -537,17 +529,30 @@ OrganizeCollectionView.prototype = {
         }
 
         // update the new name immediately
-        let path = Gtk.TreePath.new_from_string(pathStr);
         let iter = this._model.model.get_iter(path)[1];
         this._model.model.set_value(iter, OrganizeModelColumns.NAME, newText);
 
         // actually create the new collection
         let job = new CreateCollectionJob(newText);
-        job.run(Lang.bind(this, this._onNewCollectionCreated, path));
+        job.run(Lang.bind(this, this._onNewCollectionCreated));
     },
 
-    _onTextEditCanceled: function() {
-        this._model.removePlaceholder();
+    _onTextEdited: function(cell, pathStr, newText) {
+        this._onTextEditedReal(cell, Gtk.TreePath.new_from_string(pathStr), newText);
+    },
+
+    _onTextEditCanceled: function(cell) {
+        if (this._choiceConfirmed) {
+            this._choiceConfirmed = false;
+
+            let entry = this._viewCol.cell_area.get_edit_widget();
+            let path = this._model.getPlaceholder(false);
+
+            if (entry && path)
+                this._onTextEditedReal(cell, path, entry.get_text());
+        } else {
+            this._model.removePlaceholder();
+        }
     },
 
     _checkCellFunc: function(col, cell, model, iter) {
@@ -580,18 +585,20 @@ OrganizeCollectionView.prototype = {
 
         this._rendererText.editable = true;
         this.widget.set_cursor_on_cell(path, this._viewCol, this._rendererText, true);
+    },
+
+    confirmedChoice: function() {
+        this._choiceConfirmed = true;
     }
-};
+});
 
 const OrganizeCollectionDialogResponse = {
     ADD: 1
 };
 
-function OrganizeCollectionDialog(toplevel) {
-    this._init(toplevel);
-};
+const OrganizeCollectionDialog = new Lang.Class({
+    Name: 'OrganizeCollectionDialog',
 
-OrganizeCollectionDialog.prototype = {
     _init: function(toplevel) {
         this.widget = new Gtk.Dialog({ transient_for: toplevel,
                                        modal: true,
@@ -600,7 +607,7 @@ OrganizeCollectionDialog.prototype = {
                                        default_height: 250 });
 
         this.widget.add_button('gtk-add', OrganizeCollectionDialogResponse.ADD);
-        this.widget.add_button('gtk-ok', Gtk.ResponseType.OK);
+        let okButton = this.widget.add_button('gtk-ok', Gtk.ResponseType.OK);
         this.widget.set_default_response(Gtk.ResponseType.OK);
 
         let contentArea = this.widget.get_content_area();
@@ -613,6 +620,20 @@ OrganizeCollectionDialog.prototype = {
         sw.add(collView.widget);
         contentArea.add(sw);
 
+        // HACK:
+        // - We want clicking on "OK" to add the typed-in collection if we're editing.
+        // - Unfortunately, since we focus out of the editable entry in order to
+        //   click the button, we'll get an editing-canceled signal on the renderer
+        //   from GTK. As this handler will run before focus-out, we here signal the
+        //   view to ignore the next editing-canceled signal and add the collection in
+        //   that case instead.
+        //
+        okButton.connect('button-press-event', Lang.bind(this,
+            function() {
+                collView.confirmedChoice();
+                return false;
+            }));
+
         this.widget.connect('response', Lang.bind(this,
             function(widget, response) {
                 if (response == OrganizeCollectionDialogResponse.ADD)
@@ -621,13 +642,11 @@ OrganizeCollectionDialog.prototype = {
 
         this.widget.show_all();
     }
-};
+});
 
-function SelectionController() {
-    this._init();
-};
+const SelectionController = new Lang.Class({
+    Name: 'SelectionController',
 
-SelectionController.prototype = {
     _init: function() {
         this._selection = [];
         this._selectionMode = false;
@@ -686,14 +705,12 @@ SelectionController.prototype = {
     getSelectionMode: function() {
         return this._selectionMode;
     }
-};
+});
 Signals.addSignalMethods(SelectionController.prototype);
 
-function SelectionToolbar() {
-    this._init();
-}
+const SelectionToolbar = new Lang.Class({
+    Name: 'SelectionToolbar',
 
-SelectionToolbar.prototype = {
     _init: function() {
         this._itemListeners = {};
         this._insideRefresh = false;
@@ -710,7 +727,6 @@ SelectionToolbar.prototype = {
         this._leftBox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
         this._leftGroup = new Gtk.ToolItem({ child: this._leftBox });
         this.widget.insert(this._leftGroup, -1);
-        this._leftGroup.show_all();
 
         this._toolbarFavorite = new Gtk.ToggleButton({ child: new Gtk.Image ({ icon_name: 'emblem-favorite-symbolic',
                                                                                pixel_size: 32 })});
@@ -719,6 +735,7 @@ SelectionToolbar.prototype = {
 
         this._toolbarPrint = new Gtk.Button({ child: new Gtk.Image ({ icon_name: 'printer-symbolic',
                                                                       pixel_size: 32 })});
+        this._toolbarPrint.set_tooltip_text(_("Print"));
         this._leftBox.add(this._toolbarPrint);
         this._toolbarPrint.connect('clicked', Lang.bind(this, this._onToolbarPrint));
 
@@ -730,7 +747,6 @@ SelectionToolbar.prototype = {
         this._rightBox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
         this._rightGroup = new Gtk.ToolItem({ child: this._rightBox });
         this.widget.insert(this._rightGroup, -1);
-        this._rightGroup.show_all();
 
         this._toolbarCollection = new Gtk.Button({ child: new Gtk.Image ({ icon_name: 'list-add-symbolic',
                                                                            pixel_size: 32 })});
@@ -750,7 +766,7 @@ SelectionToolbar.prototype = {
         this._rightBox.add(this._toolbarOpen);
         this._toolbarOpen.connect('clicked', Lang.bind(this, this._onToolbarOpen));
 
-        this.widget.show();
+        this.widget.show_all();
 
         Global.selectionController.connect('selection-mode-changed',
                                            Lang.bind(this, this._onSelectionModeChanged));
@@ -799,8 +815,9 @@ SelectionToolbar.prototype = {
         let apps = [];
         let favCount = 0;
         let showFavorite = true;
-        let canTrash = true;
-        let showPrint = false;
+        let showTrash = true;
+        let showPrint = true;
+        let showOpen = true;
 
         this._insideRefresh = true;
 
@@ -816,27 +833,15 @@ SelectionToolbar.prototype = {
                     (apps.indexOf(doc.defaultAppName) == -1))
                     apps.push(doc.defaultAppName);
 
-                if (!doc.canTrash())
-                    canTrash = false;
+                showTrash &= doc.canTrash();
+                showPrint &= !doc.collection;
             }));
 
         showFavorite &= ((favCount == 0) || (favCount == selection.length));
+        showOpen = (apps.length > 0);
 
-        if (selection.length == 1) {
-            let urn = selection[0];
-            if (!Global.collectionManager.getItemById(urn))
-                showPrint = true;
-        }
-
-        if (showPrint)
-            this._toolbarPrint.show_all();
-        else
-            this._toolbarPrint.hide();
-
-        if (canTrash)
-            this._toolbarTrash.show_all();
-        else
-            this._toolbarTrash.hide();
+        if (selection.length > 1)
+            showPrint = false;
 
         let openLabel = null;
         if (apps.length == 1) {
@@ -846,11 +851,7 @@ SelectionToolbar.prototype = {
             // Translators: this is the Open action in a context menu
             openLabel = _("Open");
         }
-
-        if (apps.length > 0) {
-            this._toolbarOpen.set_tooltip_text(openLabel);
-            this._toolbarOpen.show_all();
-        }
+        this._toolbarOpen.set_tooltip_text(openLabel);
 
         if (showFavorite) {
             let isFavorite = (favCount == selection.length);
@@ -868,11 +869,13 @@ SelectionToolbar.prototype = {
 
             this._toolbarFavorite.reset_style();
             this._toolbarFavorite.set_tooltip_text(favoriteLabel);
-            this._toolbarFavorite.show_all();
-        } else {
-            this._toolbarFavorite.hide();
         }
 
+        this._toolbarPrint.set_visible(showPrint);
+        this._toolbarTrash.set_visible(showTrash);
+        this._toolbarOpen.set_visible(showOpen);
+        this._toolbarFavorite.set_visible(showFavorite);
+
         this._insideRefresh = false;
     },
 
@@ -971,4 +974,4 @@ SelectionToolbar.prototype = {
               },
               onCompleteScope: this });
     }
-};
+});
